//*****************************************************************************
//  Copyright (C) 2007 Microsoft Corporation
//  All rights reserved.
//*****************************************************************************

#include "Run.h"
#ifdef _MSC_VER
    #include <windows.h>
    #include <strsafe.h>
#endif
#include <iostream>
#include <fstream>
#include <stdio.h>

using namespace std;
TestSystem::Run g_run;

NITS_EXPORT bool NitsParseArgument(_In_z_ char *str)
{
    return g_run.ParseArgument(ConvertStringToW(str));
}

NITS_EXPORT int NitsExecuteRun()
{
    return g_run.Execute();
}

NITS_EXPORT int NitsCheckVersion(int version)
{
    if (TestSystem::SharedMemoryVersion == version)
    {
        return 1;
    }

    wcerr << L"ERROR: Shared memory version mismatch. Please unload and update NITS binaries.\n";
    return 0;
}

class BSTRBuffer
{
public:
    BSTRBuffer(_In_z_ const wchar_t *text)
    {
        unsigned count = wcslen(text);
        
        unsigned available = (sizeof(m_data) - sizeof(unsigned))/sizeof(wchar_t) - 1;
        if (count > available)
        {
            count = available;
        }

        wchar_t *string = GetBSTR();
        memcpy(string, text, count * sizeof(wchar_t));
        string[count] = L'\0';
        *reinterpret_cast<unsigned *>(m_data) = count * sizeof(wchar_t);
    }

    wchar_t *GetBSTR()
    {
        return reinterpret_cast<wchar_t *>(m_data + sizeof(unsigned));
    }
private:
    char m_data[256];
};

#ifdef _MSC_VER
class CWTTLogger
{

public:
    CWTTLogger()
    {
        library = LoadLibraryExW(L"WttLog.dll", NULL, 0);
        WTTLogInit = (HRESULT (WINAPI *)())GetProcAddress(library, "WTTLogInit");
        WTTLogUninit = (void (WINAPI *)())GetProcAddress(library, "WTTLogUninit");
        WTTLogCreateLogDevice = (HRESULT (WINAPI *)(_In_opt_ LPWSTR, _Out_ LONG *))GetProcAddress(library, "WTTLogCreateLogDevice");
        WTTLogCloseLogDevice = (HRESULT (WINAPI *)(_In_ LONG, _In_opt_ LPWSTR))GetProcAddress(library, "WTTLogCloseLogDevice");
        WTTLogStartTest = (HRESULT (WINAPI *)(_In_ LONG, _In_ LPWSTR))GetProcAddress(library, "WTTLogStartTest");
        WTTLogEndTest = (HRESULT (WINAPI *)(_In_ LONG, _In_ LPWSTR, _In_ DWORD, _In_opt_ LPWSTR))GetProcAddress(library, "WTTLogEndTest");

        WTTLogInit();
    }

    ~CWTTLogger()
    {
        WTTLogUninit();
        FreeLibrary(library);
    }

    HRESULT CreateLogDevice
        (
        _In_opt_ LPWSTR pwszLogDeviceName,
        _Out_    LONG* phDevice
        )
    {
        return WTTLogCreateLogDevice(pwszLogDeviceName, phDevice);
    }

    HRESULT CloseLogDevice
        (
        _In_opt_    LPWSTR  pwszLogDeviceName,
        _In_        LONG    hDevice
        )
    {
        return WTTLogCloseLogDevice(hDevice, pwszLogDeviceName);
    }

    HRESULT StartTest
        (
        _In_  LPWSTR   pwszTestName, // Name of the test case
        _In_  LONG     hDevice       // handle to the device
        )
    {
        return WTTLogStartTest(hDevice, pwszTestName);
    }

    HRESULT EndTest
        (
        _In_     LPWSTR pwszTestName, // Name of the test case
        _In_     DWORD dwResult,     // test result
        _In_opt_ LPWSTR pwszRepro,    // repro line
        _In_     LONG hDevice       // handle to the device
        )
    {
        return WTTLogEndTest(hDevice, pwszTestName, dwResult, pwszRepro);
    }
private:
    HMODULE library;
    HRESULT (WINAPI *WTTLogInit)();
    void (WINAPI *WTTLogUninit)();
    HRESULT (WINAPI *WTTLogCreateLogDevice)(_In_opt_ LPWSTR, _Out_ LONG *);
    HRESULT (WINAPI *WTTLogCloseLogDevice)(_In_ LONG, _In_opt_ LPWSTR);
    HRESULT (WINAPI *WTTLogStartTest)(_In_ LONG, _In_ LPWSTR);
    HRESULT (WINAPI *WTTLogEndTest)(_In_ LONG, _In_ LPWSTR, _In_ DWORD, _In_opt_ LPWSTR);
};

#endif

using namespace std;
using namespace TestSystem;

// injector installation/cross process injection is currently disabled on linux/unix
#ifdef _MSC_VER
bool InstallInjector()
{
    /* Check for existence of nitsinj.dll in system32. */
    char path[MAX_PATH];
    unsigned length = GetSystemDirectoryA(path, MAX_PATH);

    strcpy_s(path + length, MAX_PATH - length, "\\nitsinj.dll");

    HANDLE file = CreateFileA(path, 0, FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);
    if (file == INVALID_HANDLE_VALUE)
    {
        wcerr << L"ERROR: Injector module nitsinj.dll is not present in system32!\n";
        return false;
    }

    CloseHandle(file);

    strcpy_s(path + length, MAX_PATH - length, "\\nitsdll.dll");
    file = CreateFileA(path, 0, FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);
    if (file == INVALID_HANDLE_VALUE)
    {
        wcerr << L"ERROR: Framework module nitsdll.dll is not present in system32!\n";
        return false;
    }

    CloseHandle(file);

    /* Set registry values for AppInit_DLLs. */

    /* Open HKLM\Software\Microsoft\Windows NT\CurrentVersion\Windows */
    HKEY key;
    DWORD value;
    RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"Software\\Microsoft\\Windows NT\\CurrentVersion\\Windows", 0, KEY_SET_VALUE, &key);

    /* Set AppInit_DLLs (REG_SZ) to L"nitsinj.dll". */
    RegSetValueEx(key, L"AppInit_DLLs", 0, REG_SZ, (const BYTE *)L"nitsinj.dll", sizeof(L"nitsinj.dll"));

    /* Set LoadAppInit_DLLs (REG_DWORD) to 1. */
    value = 1;
    RegSetValueEx(key, L"LoadAppInit_DLLs", 0, REG_DWORD, (const BYTE *)&value, sizeof(DWORD));

    /* Set RequireSignedAppInit_DLLs (REG_DWORD) to 0. */
    value = 0;
    RegSetValueEx(key, L"RequireSignedAppInit_DLLs", 0, REG_DWORD, (const BYTE *)&value, sizeof(DWORD));

    RegCloseKey(key);

    /* In case this was set, clear it. */
    GetGlobals().m_unload = 0;

    wcerr << L"WARNING: Pre-existing processes are not affected by injector. To fix this, reboot.\n";
    return true;
}

void RemoveInjector()
{
    /* Remove registry values for AppInit_DLLs. */

    /* Open HKLM\Software\Microsoft\Windows NT\CurrentVersion\Windows */
    HKEY key;
    DWORD value;
    RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"Software\\Microsoft\\Windows NT\\CurrentVersion\\Windows", 0, KEY_SET_VALUE, &key);

    /* Set AppInit_DLLs (REG_SZ) to NULL. */
    RegSetValueEx(key, L"AppInit_DLLs", 0, REG_SZ, NULL, 0);

    /* Set LoadAppInit_DLLs (REG_DWORD) to 0. */
    value = 0;
    RegSetValueEx(key, L"LoadAppInit_DLLs", 0, REG_DWORD, (const BYTE *)&value, sizeof(DWORD));

    RegCloseKey(key);

    /* When the test run starts, make all the injectors unload. */
    GetGlobals().m_unload = 1;
}

#endif

Run *Run::s_run = NULL;

Run &Run::GetInstance()
{
    if (s_run == NULL)
    {
        throw Exception();
    }

    return *s_run;
}

bool Run::IsValid()
{
    return s_run != NULL;
}

Param::Param(_In_ PCWSTR name,
             _In_ PCWSTR value)
    : m_name(Copy(name)),
      m_value(Copy(value))
{
}

Param::~Param()
{
    delete [] m_name;
    delete [] m_value;
}

Module::Module(_In_ PCWSTR name)
    : m_name(Copy(name)),
      m_library(NULL)
{
}

Module::~Module()
{
    delete [] m_name;

    PAL_FreeLibrary(m_library);

    for (DWORD i = 0; i < m_runs.size(); i++)
    {
        delete [] m_runs[i];
    }
}

void Module::SetLibrary(_In_ PAL_Library *lib)
{
    if (m_library != NULL)
    {
        throw Exception();
    }

    m_library = lib;
}

void Module::AddTest(_In_ Test *test)
{
    m_tests.push_back(test);
}

void Module::AddRun(_In_ PCWSTR command)
{
    m_runs.push_back(Copy(command));
}

void Module::RunTests(size_t start)
{
    for (size_t i = start; i < m_tests.size(); i++)
    {
        Run::GetInstance().ExecuteTest(m_tests[i], NULL);
    }
}

size_t Module::Find(_In_ PCWSTR test) const
{
    for (size_t i = 0; i < m_tests.size(); i++)
    {
        if (Equals(m_tests[i]->GetName(), test))
        {
            return i;
        }
    }

    wcerr << L"ERROR: Invalid test name '" << test << L"'\n";
    return m_tests.size();
}

void Module::Execute()
{
    //Iterate through the list of commands.
    //Parse each command and run relevant tests.
    for (DWORD i = 0; i < m_runs.size(); i++)
    {
        PWSTR command = m_runs[i];
        if (command == NULL)
        {
            //Run all tests.
            RunTests(0);
        }
        else if (StartsWith(command, L"+"))
        {
            //Search for a test and run subsequent tests.
            RunTests(Find(command + 1));
        }
        else
        {
            //Run a specific test and/or variation.
            PWSTR slash = wcschr(command, L'/');
            PWSTR choices = NULL;
            if (slash)
            {
                *slash = L'\0';
                choices = slash + 1;
            }

            size_t i = Find(command);
            if (i < m_tests.size())
            {
                 Run::GetInstance().ExecuteTest(m_tests[i], choices);
            }
        }
    }
}

Run::Run()
    : m_currentTest(NULL),
      m_loadingModule(NULL),
      m_modules(NULL),
      m_params(NULL),
      m_pause(false),
      m_reset(false),
      m_debugger(NULL),
      m_binaryFilter(NULL),
      m_binaryTarget(NULL),
      m_isolation(false),
      m_child(false),
      m_fault(false),
      m_finished(false),
      m_statistics(NULL),
      m_enabled(true),
      m_flaky(false),
      m_simMode(Manual),
      m_exclusions(NULL),
      m_exclusionsCount(0)
{
    if (s_run != NULL)
    {
        throw Exception();
    }

    s_run = this;

    m_modules = new vector<Module *>;
    m_params = new vector<Param *>;
    m_bpFault = new vector<int>;
    if (m_modules == NULL || m_params == NULL || m_bpFault == NULL)
    {
        throw Exception();
    }

    PAL_ZeroMemory(&m_config, sizeof(Configuration));
}

Run::~Run()
{
    for (DWORD i = 0; i < Modules().size(); i++)
    {
        delete Modules()[i];
    }

    m_loadingModule = NULL;

    for (DWORD i = 0; i < Params().size(); i++)
    {
        delete Params()[i];
    }

    delete m_modules;
    delete m_params;
    delete m_bpFault;

    s_run = NULL;
}

PCWSTR Run::GetParam(_In_ PCWSTR name) const
{
    for (DWORD i = 0; i < Params().size(); i++)
    {
        if (PAL_wcsicmp(Params()[i]->m_name, name) == 0)
        {
            return Params()[i]->m_value;
        }
    }

    return NULL;
}

void Run::AddModule(_In_ PCWSTR name)
{
    Module *p = new Module(name);
    if (p == NULL)
    {
        throw Exception();
    }
    try
    {
        Modules().push_back(p);
    }
    catch(...)
    {
        delete p;
        throw;
    }
    m_loadingModule = p;
}

bool Run::AddParam(_In_ PCWSTR name, _In_ PCWSTR value)
{
    for (DWORD i = 0; i < Params().size(); i++)
    {
        if (Equals(Params()[i]->m_name, name))
        {
            wcerr << L"ERROR: Test parameter '" << name <<
                     L"' defined multiple times.\n";
            return false;
        }
    }

    Param *p = new Param(name, value);
    if (p == NULL)
    {
        throw Exception();
    }

    Params().push_back(p);
    return true;
}

bool Run::AddRun(_In_ PCWSTR name, _In_ PCWSTR test)
{
    if (name[0] == '\0')
        return true;

    //Find out if this module is already loaded.
    Module *match = NULL;
    for (DWORD i = 0; i < Modules().size(); i++)
    {
        if (Equals(Modules()[i]->GetName(), name))
        {
            match = Modules()[i];
            break;
        }
    }
    
    if (match == NULL)
    {
        AddModule(name);
        match = m_loadingModule;

        /* Set m_runLock so that NitsIsActivated() returns true in here. This
         * prevents the test code from thinking that NITS is inactive during
         * initialization. */
        System &system = System::GetInstance();
        Globals &globals = system.GetGlobals();
        bool setRunLock = (PAL_InterlockedCompareExchange(&globals.m_runLock, 1, 0) == 0);

        PAL_Library *lib = NULL;
#ifdef _MSC_VER
        lib = PAL_LoadLibraryW(name);
#else
        lib = PAL_LoadLibraryA(ConvertStringToA(name));
#endif

        if (setRunLock)
        {
            PAL_InterlockedCompareExchange(&globals.m_runLock, 0, 1);
        }


        if(!lib)
            return false;

        match->SetLibrary(lib);
    }

    match->AddRun(test);
    return true;
}

#ifdef _MSC_VER
DWORD WINAPI Run::_PipeThread(_In_ void *)
#else
void *Run::_PipeThread(_In_ void *)
#endif
{
    Run &run = Run::GetInstance();
    run.PipeThread();
    return 0;
}

void Run::PipeThread()
{
    System &system = System::GetInstance();
    PAL_Semaphore *event = system.GetPipeEvent();

    do {
        system.GetGlobals().DumpPipe();
        PAL_WaitForSemaphore(event, 200);
    } while (!m_finished);

    system.GetGlobals().DumpPipe();
}

static void TranslateBinaryList(
    _In_opt_ PCWSTR source,
    _Out_writes_(bytes) char *dest,
    unsigned bytes)
{
    unsigned length;
    unsigned i;

    if (source == NULL)
    {
        dest[0] = '\0';
        return;
    }

    length = wcslen(source);
#ifdef _MSC_VER
    WideCharToMultiByte(CP_THREAD_ACP, 0, source, length + 1, dest, bytes, NULL, NULL);
#else
    wcstombs(dest, source, bytes);
#endif

    for (i = 0; i < length && i < bytes; i++)
    {
        if (dest[i] == ';')
        {
            dest[i] = '\0';
        }
    }
}

static Globals g_tempGlobals;

int Run::Execute()
{
    System &system = System::GetInstance();
    Globals &globals = system.GetGlobals();
    m_statistics = globals.GetStatistics();
    PAL_Thread *pipeThread = NULL;

    if (m_config.traces == 0)
    {
        m_config.traces = NitsTraceAllTests;
    }

    if (m_config.mode == 0)
    {
        m_config.mode = NitsTestCompoundFault;
    }

    uint64 startTicks = PAL_GetCurrentTime();

    if (IsParent())
    {
        TranslateBinaryList(m_binaryFilter, globals.m_binaryFilter, sizeof(globals.m_binaryFilter));
        TranslateBinaryList(m_binaryTarget, globals.m_binaryTarget, sizeof(globals.m_binaryTarget));

        if (!globals.StartRun(m_reset))
        {
            wcerr << L"ERROR: Another test run is in progress!\n";
            return 1;
        }

#if _MSC_VER
        if (m_wtt)
        {
            wostringstream buf;
            buf << L"$LogFile:file=\"" << m_wtt << L"\",writemode=overwrite";
            m_logger = new CWTTLogger();
            m_logger->CreateLogDevice((LPWSTR)buf.str().c_str(), &m_loggerHandle);
        }
#endif

        if (m_reset)
        {
            //Reset globals.
            TranslateBinaryList(m_binaryFilter, g_tempGlobals.m_binaryFilter, sizeof(g_tempGlobals.m_binaryFilter));
            TranslateBinaryList(m_binaryTarget, g_tempGlobals.m_binaryTarget, sizeof(g_tempGlobals.m_binaryTarget));
            g_tempGlobals.StartRun();
            globals = g_tempGlobals;
        }

        pipeThread = PAL_CreateThread(_PipeThread);
        if (pipeThread == NULL)
        {
            globals.StopRun();
            wcerr << L"ERROR: Failed to create pipe thread!\n";
            return 1;
        }
    }

    //TODO: Reset XML output file here.
    //After this, everybody opens for append.

    for (DWORD i = 0; i < Modules().size(); i++)
    {
        Modules()[i]->Execute();
    }

    //TODO: Finish and close XML output file here.

    unsigned returnCode = 0;

    if (IsParent())
    {
        if (Modules().size() > 0)
            returnCode = ReportStatistics();

        if (m_pause)
        {
            SetDefaultOptions(); //Mainly used for debugger setting.
            globals.PostPipe(L"PAUSED: Press ENTER to continue...");
            fgetc(stdin);
        }

        if (Modules().size() > 0)
        {
            Statistics const &stats = globals.GetStats();
            uint64 stopTicks = PAL_GetCurrentTime();
            double seconds = float(stopTicks - startTicks) / 1000.0;
            // double stackTicks = (double)stats.stackTicks / ((double)stats.stackLookups+0.00001);
            wostringstream buf;
            buf << L"TOTAL RUNNING TIME: " << seconds << L" seconds" << endl
                << L"TOTAL FAULT ITERATIONS: " << stats.faultIterations << endl
                << L"TOTAL SHOULDFAULT CALLS: " << stats.shouldFaults << endl;
    
            //double percentage = 100.0 * (double)stats.shouldFaultIPCs / ((double)stats.shouldFaults+0.00001);
            //buf << L"TOTAL CROSS-PROCESS SHOULDFAULT CALLS: " << stats.shouldFaultIPCs << L" (" << percentage << L"%)\n"
            buf << L"TOTAL STACK PROBES: " << stats.stackLookups << endl
            //    << L"AVERAGE TICKS/STACK PROBE: " << stackTicks << endl
                << L"TOTAL FRAME PROBES: " << stats.frameLookups << endl;
    
            /*unsigned remaining = stats.frameLookups;
            unsigned target = stats.frameInsertions;
            double percentage = 100.0 * (double)target / remaining;
            buf << L"TOTAL FRAME MISSES: " << target << L"/" << remaining << L" (" << percentage << L"%)\n";
    
            remaining -= target;
            target = stats.frameHashHits;
            percentage = 100.0 * (double)target / remaining;
            buf << L"REMAINDER, HASH HITS: " << target << L"/" << remaining << L" (" << percentage << L"%)\n";
    
            for (int i = 0; i < 16; i++)
            {
                if (stats.frameHits[i] == 0)
                {
                    continue;
                }
    
                remaining -= target;
                target = stats.frameHits[i];
                percentage = 100.0 * (double)target / remaining;
                buf << L"REMAINDER, LEVEL " << i+1 << L" CACHE HITS: " << target
                    << L"/" << remaining << L" (" << percentage << L"%)\n";
            }*/
            globals.PostPipe(buf.str().c_str());
        }

        m_finished = true;
        PAL_SignalSemaphore(system.GetPipeEvent());
        PAL_WaitAndCloseThread(pipeThread);

        globals.StopRun();

#ifdef _MSC_VER
        if (m_logger)
        {
            m_logger->CloseLogDevice(NULL, m_loggerHandle);
            delete m_logger;
        }
#endif
    }

    return returnCode;
}

void Run::ExecuteTest(_In_ Test *test, _In_opt_ PCWSTR choices)
{
    m_currentTest = test;

    //Spawn a child process to run the test for us.
    //Construct the same argument list, but for this particular test.
    wostringstream buf, repro;
    if (m_debugger)
    {
        buf << m_debugger << L" ";
        repro << m_debugger << L" ";
    }

    buf << L"nits.exe -child " << test->GetModule() << L":" << test->GetName();
    repro << L"nits.exe " << test->GetModule() << L":" << test->GetName();
    if (choices)
    {
        buf << L"/" << choices;
        repro << L"/" << choices;
    }

    if (m_config.traces == NitsTraceFailedTests)
    {
        buf << L" -trace:FailedTests";
    }
    else if (m_config.traces == NitsTraceWarnedTests)
    {
        buf << L" -trace:WarnedTests";
    }
    else if (m_config.traces == NitsTraceAllTests)
    {
        buf << L" -trace:AllTests";
    }
    else if (m_config.traces == NitsTraceAsserts)
    {
        buf << L" -trace:Asserts";
    }
    else if (m_config.traces == NitsTraceWarnings)
    {
        buf << L" -trace:Warnings";
    }
    else if (m_config.traces == NitsTraceIterations)
    {
        buf << L" -trace:Iterations";
    }
    else //NitsTraceVerbose.
    {
        buf << L" -trace:Verbose";
    }

    if (m_config.mode == NitsTestSkip)
    {
        buf << L" -mode:Skip";
    }
    else if (m_config.mode == NitsTestEnable)
    {
        buf << L" -mode:Enable";
    }
    else if (m_config.mode == NitsTestIterativeFault)
    {
        buf << L" -mode:IterativeFault";
    }
    else if (m_config.mode == NitsTestStackFault)
    {
        buf << L" -mode:StackFault";
    }
    else //NitsTestCompoundFault.
    {
        buf << L" -mode:CompoundFault";
    }

    if (m_config.breakAssert)
    {
        buf << L" -bpassert";
    }

    if (m_config.skipFlakyTests)
    {
        buf << L" -skipflaky";
    }

    if (m_fault)
    {
        buf << L" -fault";
        repro << L" -fault";
    }
    if (m_debugger)
    {
        buf << L" -debug:\"" << m_debugger << L"\"";
        repro << L" -debug:\"" << m_debugger << L"\"";
    }

    vector<Param *> &params = Params();
    for (size_t i = 0; i < params.size(); i++)
    {
        buf << L" +" << params[i]->m_name << L":" << params[i]->m_value;
        repro << L" +" << params[i]->m_name << L":" << params[i]->m_value;
    }

#ifdef _MSC_VER
    //TODO: Make WTT reporting work better for variations.
    int countFailed = m_statistics[Failed] + m_statistics[Error] + m_statistics[Killed];
    int countPassed = m_statistics[Passed] + m_statistics[Faulted];
    int countSkipped = m_statistics[Skipped] + m_statistics[Omitted];


    enum WTTLOGGER_TEST_RESULT
    {
        WTTLOG_TESTCASE_RESULT_PASS      =0x1,
        WTTLOG_TESTCASE_RESULT_FAIL      =0x2,
        WTTLOG_TESTCASE_RESULT_BLOCKED   =0x3,
        WTTLOG_TESTCASE_RESULT_WARN      =0x4,
        WTTLOG_TESTCASE_RESULT_SKIPPED   =0x5
    };

    wostringstream nameBuf;
    nameBuf << test->GetModule() << L":" << test->GetName();
    BSTRBuffer testCase(nameBuf.str().c_str());
    BSTRBuffer testCaseRepro(repro.str().c_str());
    if (m_logger)
    {
        m_logger->StartTest(testCase.GetBSTR(), m_loggerHandle);
    }
#endif
#ifdef _MSC_VER
    if (m_child || !m_isolation && !test->GetIsolation())
#endif
    {
        test->Execute(choices);
    }
#ifdef _MSC_VER
    else
    {
        // isolation not implemented currently in unix
        STARTUPINFO si;
        PROCESS_INFORMATION pi;

        ZeroMemory(&si, sizeof(si));
        si.cb = sizeof(si);
        ZeroMemory(&pi, sizeof(pi));

        si.dwFlags |= STARTF_USESHOWWINDOW;
        si.wShowWindow = SW_SHOWMINNOACTIVE;


        PWSTR temp = Copy(buf.str().c_str());
        if (!CreateProcess(NULL, temp, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi))
        {
            wcerr << L"ERROR: Failed to create child process! Error=" << GetLastError() << L"\n";
            GetGlobals().SetResult(Error);
            ReportResult();
            delete [] temp;
            if (m_logger)
            {
                m_logger->EndTest(testCase.GetBSTR(), WTTLOG_TESTCASE_RESULT_FAIL, testCaseRepro.GetBSTR(), m_loggerHandle);
            }
            return;
        }

        delete [] temp;
        int timeout = test->GetTimeout() * 1000;
        if (m_debugger)
        {
            timeout = INFINITE;
        }
        if (WaitForSingleObject(pi.hProcess, timeout) == WAIT_TIMEOUT)
        {
            TerminateProcess(pi.hProcess, 1);
            GetGlobals().SetResult(Killed);
            ReportResult();
        }
        else
        {
            DWORD exitCode;
            GetExitCodeProcess(pi.hProcess, &exitCode);

            if (exitCode != 0)
            {
                GetGlobals().SetResult(Error);
                ReportResult();
            }
        }

        CloseHandle(pi.hThread);
        CloseHandle(pi.hProcess);
    }
#endif

#ifdef _MSC_VER
    long wttResult = WTTLOG_TESTCASE_RESULT_FAIL;
    if (countFailed == m_statistics[Failed] + m_statistics[Error] + m_statistics[Killed])
    {
        if (countPassed != m_statistics[Passed] + m_statistics[Faulted])
        {
            wttResult = WTTLOG_TESTCASE_RESULT_PASS;
        }
        else if (countSkipped != (m_statistics[Skipped] + m_statistics[Omitted]))
        {
            wttResult = WTTLOG_TESTCASE_RESULT_SKIPPED;
        }
    }

    if (m_logger)
    {
        m_logger->EndTest(testCase.GetBSTR(), wttResult, testCaseRepro.GetBSTR(), m_loggerHandle);
    }
#endif
}

void Run::SetDefaultOptions()
{
    Globals &globals = GetGlobals();

    m_exclusions = NULL;
    m_exclusionsCount = 0;
    m_enabled = m_config.mode >= NitsTestEnable;
    m_simMode = m_fault ? Automatic : Manual;

    globals.Reset();
    globals.SetConfiguration(m_config);
    if (m_debugger != NULL)
    {
        globals.SetDebugger(m_debugger);
    }
}

void Run::ReportResult()
{
    PCWSTR results[] = {L"[Faulted]",
                        L"[Passed] ",
                        L"[Skipped]",
                        L"[Omitted]",
                        L"[Killed] ",
                        L"[Failed] ",
                        L"[Error]  "};

    Globals &globals = GetGlobals();
    Result result = globals.GetResult();
    double seconds = globals.GetRunTime();

    m_statistics[result]++;

    if (result < Killed && globals.GetConfiguration().traces == NitsTraceFailedTests)
    {
        return;
    }

    if (result < Faulted || result > Error)
    {
        FatalError();
        return;
    }

    Test &test = GetCurrentTest();
    wostringstream buf;
    buf << L"\t" << results[result] << L" ";
    Buffer wrappedBuf(buf);
    test.PrintName(wrappedBuf, !m_isolation);
    buf << L"\t" << seconds;
    buf << L"\n";

    //TODO: Dump XML output in addition to stderr.
    globals.PostPipe(buf.str().c_str());
}

unsigned Run::ReportStatistics()
{
    int passed = m_statistics[Faulted] +
                 m_statistics[Passed];
    int failed = m_statistics[Killed] +
                 m_statistics[Failed] +
                 m_statistics[Error];
    int total = passed + failed;

    PCWSTR results[] = {L"Faulted:",
                        L"Passed: ",
                        L"Skipped:",
                        L"Omitted:",                        
                        L"Killed: ",
                        L"Failed: ",
                        L"Error:  "};


    //TODO: Include relevant summary information in XML output.
    wostringstream buf;
    buf << L"\n\nSummary:\n";
    for (int i = 0; i < ResultCount; i++)
    {
        buf << L"\t" << results[i] << L"\t" << m_statistics[i] << L"\n";
    }

    buf << L"\n"
           L"\tSuccesses:\t" << passed << L"\n"
           L"\tFailures:\t" << failed << L"\n"
           L"\tTotal:\t\t" << total << L"\n";
    
    Globals &globals = GetGlobals();
    globals.PostPipe(buf.str().c_str());

    return failed ? 1 : 0;
}

bool Run::IsSiteExcluded(int site) const
{
    for (int i = 0; i < m_exclusionsCount; i++)
    {
        //ignore everything if exclusions is zero
        if (m_exclusions[i] == 0)
        {
            return true;
        }

        //check for specific exclusion
        if (m_exclusions[i] == site)
        {
            return true;
        }
    }
    
    return false;
}

void Run::SetExclusions(_In_reads_opt_(count) int sites[], int count)
{
    m_exclusions = sites;
    m_exclusionsCount = count;
}

bool Run::ParseArgument(_In_ PWSTR arg)
{
    PWSTR temp = wcschr(arg, L':');
    PWSTR second = NULL;

    if (temp)
    {
        *temp = L'\0';
        second = temp + 1;
    }

    if (EqualsCI(arg, L"-bpassert"))
    {
        m_config.breakAssert = true;
    }
    else if (EqualsCI(arg, L"-skipflaky"))
    {
        m_config.skipFlakyTests = true;
    }
    else if (EqualsCI(arg, L"-bpfault"))
    {
        m_config.breakFault = true;
        if (second == NULL)
        {
            wcerr << L"ERROR: 'bpfault' option requires an iteration number!\n";
            return false;
        }

        m_bpFault->push_back(PAL_wtoi(second));
    }
    else if (EqualsCI(arg, L"-debug"))
    {
        m_debugger = Copy(second);
    }
    else if (EqualsCI(arg, L"-file"))
    {
        //Open file, tokenize, and parse contents.
        if (second == NULL)
        {
            wcerr << L"ERROR: 'file' option requires a filename!\n";
            return false;
        }

        char buf[MAX_PATH];
#ifdef _MSC_VER
        WideCharToMultiByte(CP_THREAD_ACP, 0, 
                second, wcslen(second) + 1, buf, sizeof(buf), NULL, NULL);
#else
        wcstombs(buf, second, MAX_PATH);
#endif

        ifstream file(buf);
        if (!file.good())
        {
            wcerr << L"ERROR: file '" << second << L"' could not be loaded!\n";
            return false;
        }
        while (file.good())
        {
            wchar_t widebuf[MAX_PATH];
            file >> buf;
#ifdef _MSC_VER
#pragma prefast(suppress:26035)
            int size = MultiByteToWideChar(CP_THREAD_ACP, 0,
                    buf, -1, widebuf, sizeof(widebuf)/sizeof(wchar_t));
#else
            int size = mbstowcs(widebuf, buf, MAX_PATH);
#endif
            if (size == 0)
            {
                throw Exception();
            }

            if (!ParseArgument(widebuf))
                return false;
        }
    }
    else if (EqualsCI(arg, L"-filter"))
    {
        m_binaryFilter = Copy(second);
    }
    else if (EqualsCI(arg, L"-install"))
    {
#ifdef _MSC_VER
        if (!InstallInjector())
            return false;
#endif
    }
    else if (EqualsCI(arg, L"-mode"))
    {
        if (!second)
        {
            wcerr << L"ERROR: 'mode' option requires a mode to be selected!\n";
            return false;
        }
        else if (EqualsCI(second, L"Skip"))
        {
            m_config.mode = NitsTestSkip;
        }
        else if (EqualsCI(second, L"Enable"))
        {
            m_config.mode = NitsTestEnable;
        }
        else if (EqualsCI(second, L"IterativeFault"))
        {
            m_config.mode = NitsTestIterativeFault;
        }
        else if (EqualsCI(second, L"StackFault"))
        {
            m_config.mode = NitsTestStackFault;
        }
        else if (EqualsCI(second, L"CompoundFault"))
        {
            m_config.mode = NitsTestCompoundFault;
        }
        else
        {
            wcerr << L"ERROR: Invalid value for 'mode' option!\n";
            return false;
        }
    }
    else if (EqualsCI(arg, L"-pause"))
    {
        m_pause = true;
    }
    else if (EqualsCI(arg, L"-reset"))
    {
        m_reset = true;
    }
    else if (EqualsCI(arg, L"-target"))
    {
        m_binaryTarget = Copy(second);
    }
    else if (EqualsCI(arg, L"-trace"))
    {
        if (!second || EqualsCI(second, L"Verbose"))
        {
            m_config.traces = NitsTraceVerbose;
        }
        else if (EqualsCI(second, L"FailedTests"))
        {
            m_config.traces = NitsTraceFailedTests;
        }
        else if (EqualsCI(second, L"WarnedTests"))
        {
            m_config.traces = NitsTraceWarnedTests;
        }
        else if (EqualsCI(second, L"AllTests"))
        {
            m_config.traces = NitsTraceAllTests;
        }
        else if (EqualsCI(second, L"Asserts"))
        {
            m_config.traces = NitsTraceAsserts;
        }
        else if (EqualsCI(second, L"Warnings"))
        {
            m_config.traces = NitsTraceWarnings;
        }
        else if (EqualsCI(second, L"Iterations"))
        {
            m_config.traces = NitsTraceIterations;
        }
        else
        {
            wcerr << L"ERROR: Invalid value for 'trace' option!\n";
            return false;
        }
    }
    else if (Equals(arg, L"-uninstall"))
    {
#ifdef _MSC_VER
        RemoveInjector();
#endif
    }
    //V1 compatibility options.
    else if (Equals(arg, L"-assert"))
    {
    }
    else if (Equals(arg, L"-noassert"))
    {
        m_config.traces = NitsTraceAllTests;
    }
    else if (Equals(arg, L"-child"))
    {
        m_child = true;
    }
    else if (Equals(arg, L"-fault"))
    {
        m_fault = true;

        if (second)
        {
            m_config.breakFault = true;
            m_bpFault->push_back(PAL_wtoi(second));
        }
    }
    else if (Equals(arg, L"-isolate"))
    {
        m_isolation = true;
    }
    else if (Equals(arg, L"-output"))
    {
        wcerr << L"ERROR: 'output' option is not implemented!\n";
        return false;
    }
    else if (Equals(arg, L"-skip"))
    {
        m_config.mode = NitsTestSkip;
    }
    else if (Equals(arg, L"-verbose"))
    {
        m_config.traces = NitsTraceVerbose;
    }
    else if (Equals(arg, L"-noverbose"))
    {
    }
    else if (Equals(arg, L"-wtt"))
    {
        if (second == NULL)
        {
            wcerr << L"ERROR: 'wtt' option requires a filename!\n";
            return false;
        }

        m_wtt = second;
    }
    else if (StartsWith(arg, L"+"))
    {
        //Spurious prefast warning.
        if (wcslen(arg) < 1)
            throw Exception();
        AddParam(arg+1, second);
    }
    else if (!AddRun(arg, second))
    {
        //Error message already printed.
        return false;
    }

    return true;
}
