/*
 * Copyright (C) 2011 Microsoft Corporation
 * All rights reserved.
 */
#ifdef _MSC_VER
    #include <sal.h>
#endif
#include "stdlib.h"
#include "string.h"
#include "../base/Globals.h"

#include "../portability/PortableDecl.h"

#include <iostream>
using namespace std;

#ifdef _MSC_VER
# define CDECLAPI __cdecl
#else
# define CDECLAPI
#endif

extern "C" NITS_DLLIMPORT int NitsExecuteRun();
extern "C" NITS_DLLIMPORT int NitsParseArgument(_In_z_ char *str);
extern "C" NITS_DLLIMPORT int NitsCheckVersion(int version);

static void PrintUsage()
{
    cout <<      "NITS Integrated Test System PROTOTYPE v0.5\n"
                 "------------------------------------------\n\n"
                 "Usage: nits [option|test]*\n"
                 "\nV2 Options:\n"
                 "\t-bpassert\t\tBreaks on all failed asserts.\n"
                 "\t-bpfault:number\t\tBreaks on a fault injection iteration.\n"
                 "\t-debug:name\t\tAttach debugger to child processes.\n"
                 "\t-file:name.txt\t\tAppends file to command line.\n"
                 "\t-filter:binaries\tInjector must find one of these binaries (semicolon-separated).\n"
                 "\t-install\t\tAdds nitsinj.dll injection to all processes.\n"
                 "\t-mode:Skip\t\tExplores test tree and skips everything.\n"
                 "\t-mode:Enable\t\tRuns without fault injection.\n"
                 "\t-mode:IterativeFault\tFaults all single-fault locations.\n"
                 "\t-mode:StackFault\tOptimized single-fault scenarios.\n"
                 "\t-mode:CompoundFault\t(Default) Multiple-fault scenarios.\n"
                 "\t-pause\t\t\tWaits for input after running.\n"
                 "\t-reset\t\t\tIgnores a test run in progress.\n"
                 "\t-skipflaky\t\tSkips executing flaky tests.\n"
                 "\t-target:binaries\tDynamic injection on specific binaries (semicolon-separated).\n"
                 "\t-trace:FailedTests\tFailed tests only.\n"
                 "\t-trace:WarnedTests\tShows failed and warned tests.\n"
                 "\t-trace:AllTests\t\t(Default) Shows all test cases.\n"
                 "\t-trace:Asserts\t\tShows failed asserts.\n"
                 "\t-trace:Warnings\t\tShows failed asserts/warnings.\n"
                 "\t-trace:Iterations\tShows fault injection iterations.\n"
                 "\t-trace:Verbose\t\tShows product and other traces.\n"
                 "\t-uninstall\t\tRemoves nitsinj.dll injection to all processes.\n"
                 "\nV1 Compatibility Options:\n"
                 "\t-assert\t\tShows failed asserts. Default.\n"
                 "\t-noassert\t\tHides failed asserts.\n"
                 "\t-child\t\tInternal use only.\n"
                 "\t-fault\t\tAutomatic fault simulation.\n"
                 "\t-fault:number\tBreaks at a fault sim iteration.\n"
                 "\t-isolate\tIsolates tests in child processes.\n"
                 "\t-output:file\tDumps XML output to file.\n"
                 "\t-skip\t\tValidates setup/cleanup code only.\n"
                 "\t-trace\t\tShows test traces.\n"
                 "\t-verbose\tShows successful tests. Default.\n"
                 "\t-noverbose\tHides successful tests.\n"
                 "\t-wtt:file\tLogs WTT results to a file.\n"
                 "\t+name:value\tSets test parameter 'name' to 'value'.\n"
                 "\nTests:\n"
                 "\tmodule.dll\t\tRuns all tests in a module.\n"
                 "\tmodule.dll:+test\t\tRuns all tests starting with a specific test.\n"
                 "\tmodule.dll:test\t\tRuns all variations of a specific test.\n"
                 "\tmodule.dll:test/choices\tRuns a specific test variation.\n"
                 "\nResults:\n"
                 "\tFaulted:\tPassed with fault simulation.\n"
                 "\tPassed: \tPassed without fault simulation.\n"
                 "\tSkipped:\tThe test body was not run.\n"
                 "\tKilled: \tIsolation mode: the test was killed.\n"
                 "\t\t\tNOTE: Killed tests may fail to clean up properly.\n"
                 "\tFailed: \tTest assertions failed while running the test.\n"
                 "\tError:  \tSetup/cleanup failed, or no asserts were attempted.\n";
}

int CDECLAPI main(int argc, _In_reads_(argc) char *argv[])
{
    if (!NitsCheckVersion(TestSystem::SharedMemoryVersion))
    {
        return 1;
    }

    if (argc == 1)
    {
        PrintUsage();
        return 0;
    }

    for (int count = 1; count < argc; count++)
        if (!NitsParseArgument(argv[count]))
            return 1;

    return NitsExecuteRun();
}
