#include "PortableDecl.h"

#ifdef _MSC_VER
    #include <windows.h>
    #include <strsafe.h>
#endif

#include <iostream>

using namespace std;

#ifndef _MSC_VER
    #include <sys/time.h>
    #include <sys/types.h>
    #include <dlfcn.h>
#endif

// The functions in this file will not exist once PAL is ready; these are just stubs to get things to compile and provide some quick/bare-minimum implementations to get things running for now
// so I am not worrying about OACR errors here; just disabling them
#ifdef _MSC_VER
    #pragma prefast(disable: 28752)
    #pragma prefast(disable: 26006) 
    #pragma prefast(disable: 26007) 
    #pragma prefast(disable: 22104) 
#endif

PAL_Thread *PAL_CreateThread(PortableThreadProc threadProc)
{
    PAL_Thread *palThread = (PAL_Thread *)malloc(sizeof(PAL_Thread));
    if (palThread == NULL)
        return palThread;
#ifdef _MSC_VER
    palThread->m_handle = CreateThread(NULL, 0, threadProc, NULL, 0, NULL);
    if(palThread->m_handle == NULL)
    {
        wcerr << L"ERROR: Failed to create thread!\n";
    }
    return palThread;
#else
    if(pthread_create(&(palThread->m_handle), 0,threadProc, 0) != 0)
    {
        free(palThread);
        palThread = 0;
    }
    return palThread;
#endif
}

void PAL_WaitAndCloseThread(PAL_Thread *palThread)
{
    if(palThread->m_handle)
    {
#ifdef _MSC_VER
        WaitForSingleObject(palThread->m_handle, INFINITE);
        CloseHandle(palThread->m_handle);
#else
        pthread_join(palThread->m_handle, 0);
#endif
    }
}

#ifdef _MSC_VER
static SECURITY_DESCRIPTOR g_SecurityDescriptor = {
    SECURITY_DESCRIPTOR_REVISION,
    0,
    SE_DACL_PRESENT,
    NULL,
    NULL,
    NULL,
    NULL,
};

static SECURITY_ATTRIBUTES g_SecurityAttributes = {
    sizeof(SECURITY_ATTRIBUTES),
    &g_SecurityDescriptor,
    FALSE,
};
#endif

PAL_Semaphore *PAL_CreateSemaphore(const char *name)
{
    PAL_Semaphore *semaphore = (PAL_Semaphore *)malloc(sizeof(PAL_Semaphore));
    if(!semaphore)
        return 0;

#ifdef _MSC_VER


    semaphore->m_handle = CreateSemaphoreA(&g_SecurityAttributes, 0, 1000, name);

    if(!semaphore->m_handle)
    {
        free(semaphore);
        return 0;
    }
#else
    semaphore->m_handle = sem_open(name, O_CREAT);
    if(semaphore->m_handle == SEM_FAILED)
    {
        free(semaphore);
        return 0;
    }

#endif
    return semaphore;
}

PAL_Semaphore *PAL_OpenSemaphore(const char *name)
{
    PAL_Semaphore *semaphore = (PAL_Semaphore *)malloc(sizeof(PAL_Semaphore));
    if(!semaphore)
        return 0;
#ifdef _MSC_VER
    semaphore->m_handle = OpenSemaphoreA(SEMAPHORE_ALL_ACCESS, FALSE, name);

    if(!semaphore->m_handle)
    {
        free(semaphore);
        return 0;
    }
#else
#endif
    return semaphore;
}

void PAL_WaitForSemaphore(PAL_Semaphore *semaphore, DWORD timeout)
{
#ifdef _MSC_VER
    WaitForSingleObject(semaphore->m_handle, timeout);
#else
    timeout = 0;
    sem_wait(semaphore->m_handle);
#endif
}

void PAL_SignalSemaphore(PAL_Semaphore *semaphore)
{
#ifdef _MSC_VER
    ReleaseSemaphore(semaphore->m_handle, 1, NULL);
#else
    sem_post(semaphore->m_handle);
#endif
}

void PAL_DestructSemaphore(PAL_Semaphore *semaphore)
{
#ifdef _MSC_VER
    if(semaphore->m_handle)
        CloseHandle(semaphore->m_handle);
    free(semaphore);
#else

#endif
}

PAL_Library *PAL_LoadLibraryA(_In_ PCSTR name)
{
    PAL_Library *library = (PAL_Library *)malloc(sizeof(PAL_Library));

    if(library == 0)
        return 0;
#ifdef _MSC_VER
    HMODULE handle = LoadLibraryExA(name, NULL, 0);
    if (handle == NULL)
    {
        DWORD error = GetLastError();
        wcerr << L"ERROR: Could not load " << name << L", error " << error << L".\n";
        return 0;
    }
    library->m_library = handle;
    return library;
#else
    library->m_library = dlopen(name, RTLD_LAZY);
    return library;
#endif
}

PAL_Library *PAL_LoadLibraryW(_In_ PCWSTR name)
{
    PAL_Library *library = (PAL_Library *)malloc(sizeof(PAL_Library));

    if(library == 0)
        return 0;
#ifdef _MSC_VER
    HMODULE handle = LoadLibraryExW(name, NULL, 0);
    if (handle == NULL)
    {
        DWORD error = GetLastError();
        wcerr << L"ERROR: Could not load " << name << L", error " << error << L".\n";
        return 0;
    }
    library->m_library = handle;
    return library;
#else
    return 0;
#endif
}

void *PAL_GetProcAddress(PAL_Library *library, PCSTR name)
{
#ifdef _MSC_VER
    return GetProcAddress(library->m_library, name);
#else
    return dlsym(library->m_library, name);
#endif
}

void PAL_FreeLibrary(PAL_Library *loadedLibrary)
{
    if(loadedLibrary == 0)
        return;

    if (loadedLibrary->m_library)
    {
#ifdef _MSC_VER
        FreeLibrary(loadedLibrary->m_library);
#else
        dlclose(loadedLibrary->m_library);
#endif
    }
}

void PAL_Sleep(uint64 msec)
{
#if defined(_MSC_VER)
    Sleep((DWORD)msec);
#else
    struct timespec ts;
    ts.tv_sec = (long)(msec/1000);
    ts.tv_nsec = (long)((msec%1000)*1000*1000);
    nanosleep(&ts, NULL);
#endif
}

uint64 PAL_GetCurrentTime()
{
#if defined(_MSC_VER)
    return GetTickCount64();
#else
    struct timeval tv;
    struct timezone tz;
    memset(&tv, 0, sizeof(tv));
    memset(&tz, 0, sizeof(tz));

    if (gettimeofday(&tv, &tz) != 0)
        return 0;

    return ((uint64)tv.tv_sec * (uint64)1000000 + (uint64)tv.tv_usec);
#endif
}

long PAL_InterlockedIncrement(volatile long *target)
{
#if defined(_MSC_VER)
    return InterlockedIncrement(target);
#else
    // TODO: PAL will define this; adding a simple stub here
    *target = *target + 1;
    return *target;
#endif
}

long PAL_InterlockedExchange(volatile long *target, long value)
{
#if defined(_MSC_VER)
    return InterlockedExchange(target, value);
#else
    // TODO: PAL will define this; adding a simple stub here
    long temp = *target;
    *target = value;
    return temp;
#endif
}

long PAL_InterlockedCompareExchange(volatile long *destination, long exchange, long comparand)
{
#if defined(_MSC_VER)
    return InterlockedCompareExchange(destination, exchange, comparand);
#else
    // TODO: PAL will define this; adding a simple stub here
    long temp = *destination;
    if(*destination == comparand)
    {
        *destination = exchange;
    }
    return temp;
#endif
}

void *PAL_InterlockedCompareExchangePointer(void * volatile *destination, void *exchange, void *comparand)
{
#if defined(_MSC_VER)
    return InterlockedCompareExchangePointer(destination, exchange, comparand);
#else
    // this will be replaced by PAL implementation
    void *temp = *destination;
    if(*destination == comparand)
    {
        *destination = exchange;
    }
    return temp;
#endif
}

int PAL_wtoi(_In_ wchar_t *str)
{
#if defined(_MSC_VER)
    return _wtoi(str);
#else
    return wcstoul(str, 0, 10);
#endif
}

int PAL_stricmp(_In_ const char *string1, _In_ const char *string2)
{
#if defined(_MSC_VER)
    return _stricmp(string1, string2);
#else
    return strcasecmp(string1, string2);
#endif

}

int PAL_wcsicmp(_In_ const wchar_t *string1, _In_ const wchar_t *string2)
{
#if defined(_MSC_VER)
    return _wcsicmp(string1, string2);
#else
    return wcscasecmp(string1, string2);
#endif
}

int PAL_strnicmp(_In_ const char *string1, _In_ const char *string2, size_t n)
{
#if defined(_MSC_VER)
    return _strnicmp(string1, string2, n);
#else
    return strncasecmp(string1, string2, n);
#endif
}

int PAL_wcsnicmp(_In_ const wchar_t *string1, _In_ const wchar_t *string2, size_t n)
{
#if defined(_MSC_VER)
    return _wcsnicmp(string1, string2, n);
#else
    return wcsncasecmp(string1, string2, n);
#endif
}

int PAL_CopyByLenW(_Out_writes_z_(length) wchar_t *dest, _In_ const wchar_t *source, size_t length)
{
#ifdef _MSC_VER
    return (FAILED(StringCchCopy(dest, length, source)) == FALSE);
#else
    wcsncpy(dest, source, length);
    return 0;
#endif
}

int PAL_ConcatByLenW(_Inout_updates_z_(length) wchar_t *dest, _In_ const wchar_t *source, size_t length)
{
#ifdef _MSC_VER
    return wcscat_s(dest, length, source);
#else
    wcsncat(dest, source, length);
    return 0;
#endif
}

int PAL_CopyByLenA(_Out_writes_z_(length) char *dest, _In_ const char *source, size_t length)
{
#ifdef _MSC_VER
    return (FAILED(StringCchCopyA(dest, length, source)) == FALSE);
#else
    strncpy(dest, source, length);
    return 0;
#endif
}



DWORD PAL_GetCurrentProcessId()
{
#ifdef _MSC_VER
    return GetCurrentProcessId();
#else
    return getpid();
#endif
}

DWORD PAL_GetCurrentThreadId()
{
#ifdef _MSC_VER
    return GetCurrentThreadId();
#else
    return pthread_self();
#endif
}

void PAL_ZeroMemory(void *dest, size_t length)
{
#ifdef _MSC_VER
    ZeroMemory(dest, length);
#else
    memset(dest, 0, length);
#endif
}

void PAL_CopyMemory(void *dest, const void *source, size_t length)
{
#ifdef _MSC_VER
    CopyMemory(dest, source, length);
#else
    memcpy(dest, source, length);
#endif
}

PAL_SharedMemory *PAL_SharedMemory_Open(PCWSTR name, size_t size)
{
    PAL_SharedMemory *sharedMemory = (PAL_SharedMemory *)malloc(sizeof(PAL_SharedMemory));

    if(sharedMemory == 0)
        return 0;

#ifdef _MSC_VER
    sharedMemory->m_handle = CreateFileMapping(INVALID_HANDLE_VALUE,
                                      &g_SecurityAttributes,
                                      PAGE_READWRITE,
                                      0,
                                      size,
                                      name);
    if (sharedMemory->m_handle == NULL)
    {
        free(sharedMemory);
        return 0;
    }
#else
#endif
    return sharedMemory;
}

void *PAL_SharedMemory_Map(PAL_SharedMemory *sharedMemory, size_t size)
{
#ifdef _MSC_VER
    return MapViewOfFile(sharedMemory->m_handle,
                        FILE_MAP_ALL_ACCESS, 0, 0, size);
#else
    // this is just a placeholder; PAL implementation will replace it
    return malloc(size);
#endif
}

void PAL_SharedMemory_Unmap(void *mappedRegion)
{
#ifdef _MSC_VER
    UnmapViewOfFile(mappedRegion);
#else
    free(mappedRegion);
#endif
}

void PAL_SharedMemory_Close(PAL_SharedMemory *sharedMemory)
{
#ifdef _MSC_VER
    CloseHandle(sharedMemory->m_handle);
#else
#endif
    free(sharedMemory);
}
