#ifndef __PORTABLE_DECL_H__
#define __PORTABLE_DECL_H__

#include "stdlib.h"
#include "string.h"

#ifdef _MSC_VER
    #include <windef.h>
#else
    #include <fcntl.h>
    #include <semaphore.h>
    #include <pthread.h>
#endif

#ifdef __cplusplus
    #define NITS_EXTERN_C extern "C"
#else
    #define NITS_EXTERN_C extern
#endif /* __cplusplus */

// this may affect all the dirs so I think I should keep this commented out; this is just a note that we should not use SAL v1 notations
// I have compiled the code with these uncommented and verified that none of the following undef'd notations are being used in nits directory
/*
// SAL v1 notations which we need to stop using for two reasons 
// 1. OACR team recommends SAL v2 and 
// 2. on linux some of the notations like __in, __out give trouble since some std library implementations have these as variable names
#ifdef __in
#undef __in
#endif

#ifdef __out
#undef __out
#endif

#ifdef __in_opt
#undef __in_opt
#endif

#ifdef __inout
#undef __inout
#endif

#ifdef __in_z
#undef __in_z
#endif

#ifdef __in_z_opt
#undef __in_z_opt
#endif

#ifdef __in_ecount
#undef __in_ecount
#endif

#ifdef __in_ecount_opt
#undef __in_ecount_opt
#endif

#ifdef __in_bcount
#undef __in_bcount
#endif

#ifdef __deref_out_bcount
#undef __deref_out_bcount
#endif

#ifdef __out_ecount
#undef __out_ecount
#endif
*/

// SAL v2 notations we need to use

#ifndef _In_
    #define _In_
#endif

#ifndef _In_z_
    #define _In_z_
#endif

#ifndef _Out_
    #define _Out_
#endif

#ifndef _In_opt_
    #define _In_opt_
#endif

#ifndef _In_opt_z_
    #define _In_opt_z_
#endif

#ifndef _Inout_
    #define _Inout_
#endif

#ifndef _In_reads_
    #define _In_reads_(count)
#endif

#ifndef _In_reads_opt_
    #define _In_reads_opt_(count)
#endif

#ifndef _In_reads_bytes_
    #define _In_reads_bytes_(n)
#endif

#ifndef _Outptr_result_bytebuffer_
    #define _Outptr_result_bytebuffer_(n)
#endif

#ifndef _Out_writes_
    #define _Out_writes_(n)
#endif

#ifndef _MSC_VER
    typedef unsigned long DWORD, *PDWORD, *LPDWORD;
    typedef int BOOL;
    typedef unsigned long HRESULT;
    typedef const wchar_t *PCWSTR;
    typedef wchar_t *PWSTR;
    typedef const char *PCSTR;
    typedef char *PSTR;
    typedef void *PVOID;
    typedef PVOID HANDLE;
    #define NO_ERROR 0
    #define INFINITE 0xFFFFFFFF
    #define WINAPI
    #define S_OK 0
    #define INT_MAX 2147483647
    #define ERROR_GEN_FAILURE 31
    #define TRUE 1
    #define FALSE 0
#endif

#ifdef _MSC_VER
    #ifndef WIN32_FROM_HRESULT
        #define WIN32_FROM_HRESULT(hr) (HRESULT_FACILITY(hr) == FACILITY_WIN32 ? HRESULT_CODE(hr) : hr) 
    #endif
#else
    #define WIN32_FROM_HRESULT(hr) hr
    #define HRESULT_FROM_WIN32(error) error
#endif

#ifdef _MSC_VER
    #define NO_INLINE __declspec(noinline)
#else
    #define NO_INLINE __attribute__((noinline))
#endif

#ifdef _MSC_VER
    typedef unsigned __int64 uint64;
#else
    typedef unsigned long long uint64;
#endif

typedef struct _PAL_Thread
{
#ifdef _MSC_VER
    HANDLE m_handle;
#else
    pthread_t m_handle;
#endif
}
PAL_Thread;

#ifdef _MSC_VER
    typedef DWORD (*PortableThreadProc)(_In_ void *);
#else
    typedef void *(*PortableThreadProc)(_In_ void *);
#endif

NITS_EXTERN_C PAL_Thread *PAL_CreateThread(PortableThreadProc threadProc);
NITS_EXTERN_C void PAL_WaitAndCloseThread(PAL_Thread *palThread);


typedef struct _PAL_Semaphore
{
#ifdef _MSC_VER
    HANDLE m_handle;
#else
    sem_t *m_handle;
#endif
}
PAL_Semaphore;

NITS_EXTERN_C PAL_Semaphore *PAL_CreateSemaphore(const char *name);
NITS_EXTERN_C PAL_Semaphore *PAL_OpenSemaphore(const char *name);
NITS_EXTERN_C void PAL_WaitForSemaphore(PAL_Semaphore *semaphore, DWORD timeout);
NITS_EXTERN_C void PAL_SignalSemaphore(PAL_Semaphore *semaphore);
NITS_EXTERN_C void PAL_DestructSemaphore(PAL_Semaphore *semaphore);

typedef struct _PAL_Library
{
#ifdef _MSC_VER
    HMODULE m_library;      //Freed after the test run.
#else
    void *m_library;
#endif
}
PAL_Library;

NITS_EXTERN_C PAL_Library *PAL_LoadLibraryA(_In_ PCSTR name);
NITS_EXTERN_C PAL_Library *PAL_LoadLibraryW(_In_ PCWSTR name);
NITS_EXTERN_C void *PAL_GetProcAddress(PAL_Library *library, PCSTR name);
NITS_EXTERN_C void PAL_FreeLibrary(PAL_Library *loadedLibrary);

typedef struct _PAL_SharedMemory
{
#ifdef _MSC_VER
    HANDLE m_handle;
#else

#endif
}
PAL_SharedMemory;

NITS_EXTERN_C PAL_SharedMemory *PAL_SharedMemory_Open(PCWSTR name, size_t size);
NITS_EXTERN_C void PAL_SharedMemory_Close(PAL_SharedMemory *sharedMemory);
NITS_EXTERN_C void *PAL_SharedMemory_Map(PAL_SharedMemory *sharedMemory, size_t size);
NITS_EXTERN_C void PAL_SharedMemory_Unmap(void *mappedRegion);

NITS_EXTERN_C void PAL_Sleep(uint64 msec);
NITS_EXTERN_C uint64 PAL_GetCurrentTime();
NITS_EXTERN_C long PAL_InterlockedIncrement(volatile long *target);
NITS_EXTERN_C long PAL_InterlockedExchange(volatile long *target, long value);
NITS_EXTERN_C long PAL_InterlockedCompareExchange(volatile  long *destination, long exchange, long comparand);
NITS_EXTERN_C void *PAL_InterlockedCompareExchangePointer(void * volatile *destination, void *exchange, void *comparand);
NITS_EXTERN_C int PAL_stricmp(_In_ const char *string1, _In_ const char *string2);
NITS_EXTERN_C int PAL_wcsicmp(_In_ const wchar_t *string1, _In_ const wchar_t *string2);
NITS_EXTERN_C int PAL_strnicmp(_In_ const char *string1, _In_ const char *string2, size_t n);
NITS_EXTERN_C int PAL_wcsnicmp(_In_ const wchar_t *string1, _In_ const wchar_t *string2, size_t n);
NITS_EXTERN_C int PAL_wtoi(_In_ wchar_t *str);
NITS_EXTERN_C int PAL_CopyByLenW(_Out_writes_z_(length) wchar_t *dest, _In_ const wchar_t *source, size_t length);
NITS_EXTERN_C int PAL_CopyByLenA(_Out_writes_z_(length) char *dest, _In_ const char *source, size_t length);
NITS_EXTERN_C int PAL_ConcatByLenW(_Inout_updates_z_(length) wchar_t *dest, _In_ const wchar_t *source, size_t length);
NITS_EXTERN_C DWORD PAL_GetCurrentProcessId();
NITS_EXTERN_C DWORD PAL_GetCurrentThreadId();
NITS_EXTERN_C void PAL_ZeroMemory(void *dest, size_t length);
NITS_EXTERN_C void PAL_CopyMemory(void *dest, const void *source, size_t length);


#ifndef _MSC_VER
    #define ERROR_INVALID_PARAMETER 87
    #define ERROR_OUTOFMEMORY 14
    #define MAX_PATH 0x00000104
#endif

#endif

