//*****************************************************************************
// Copyright (C) 2003 Microsoft Corporation
// All rights reserved.
//*****************************************************************************

#ifndef HOOK_BUILD
    #define HOOK_BUILD
#endif

#ifdef _MSC_VER
    #include <windows.h>
#endif

#include <nits.h>

extern "C" {
#include "Product.h"
}

using namespace TestSystem;

// for now commenting the runtime trapping tests;
// TODO: GIRISH Re-enable these

NitsTrapTable(MyTrap, 0)
    void (NITS_CALL *SampleFunc)(unsigned);
NitsEndTrapTable

void NITS_CALL originalFunc(unsigned x)
{
    if (x == 0)
        NitsTrace(L"Before mocking: NitsGetTrap called originalFunc");
    else if (x == 1)
        NitsTrace(L"Before mocking: MyTrap.SampleFunc called originalFunc");
    else if (x == 3)
        NitsTrace(L"After revert: MyTrap.SampleFunc called originalFunc");
    else if (x == 4)
        NitsTrace(L"During mocking: mockFunc called originalFunc through NitsLastTrap");
    else
        NitsAssert(false, L"unexpected x value");
}

void NITS_CALL mockFunc(unsigned x)
{
    if (x == 2)
        NitsTrace(L"During mocking: MyTrap.SampleFunc called mockFunc");
    else
        NitsAssert(false, L"unexpected x value");
    //Not yet implemented.
    //NitsLastTrap(MyTrap, SampleFunc)(4);
}

NitsTrapValue(MyTrap)
    originalFunc,
NitsEndTrapValue

TTEST(AuditTest)(ITest &test)
{
    int caseNumber = 2;
    DWORD error = MakeReports(L"MyCompany", caseNumber);
    NitsCompare(error, NO_ERROR, L"MakeReports should succeed");

    error = BalanceBooks(L"MyCompany", caseNumber);
    NitsCompare(error, NO_ERROR, L"BalanceBooks should succeed");

    //In a real product there would be additional validation.
#ifdef _MSC_VER
    PCSTR testDll = "NitsSample.dll";
#else
    PCSTR testDll = "libnitssample.so";
#endif

    NitsTrapHandle h = NitsOpenTrap(testDll, MyTrap);
    
    //Retrieves MyTrap.SampleFunc with an embedded version check.
    NitsGetTrap(h, MyTrap, SampleFunc)(0);

    //Direct call through the trap. Mockable product code does this.
    MyTrap.SampleFunc(1);

    //Changes the behavior of SampleFunc.
    //NOTE: Never change a table directly - it might be read-only memory!
    NitsSetTrap(h, MyTrap, SampleFunc, mockFunc);

    //Mockable product code calls again, but this time the path is different!
    MyTrap.SampleFunc(2);

    //Reverses the effect of NitsSetTrap.
    NitsCloseTrap(h);

    //Back to the behavior we saw before.
    MyTrap.SampleFunc(3);

    NitsCopyTrap(MyTrap, testDll, testDll);
}

void Body(Switch &test)
{
    NitsTrace(L"trace test!");
    NitsAssert(1 == 1, L"assert test!");
    NitsCompare(1, 1, L"compare test!");
    NitsCompareString(L"A", L"A", L"string test!");
    NitsCompareSubstring(L"ABC", L"B", L"substring test!");

    NitsCallSite s = TSITE(5);

    if (NitsShouldFault(s, NitsAutomatic))
    {
        NitsAssert(false, L"fault sim");
    }
}

TGROUP(Group1)(Switch &test)
{
    NitsTrace(L"Group1!");
}

TGROUP(Group2)(Switch &test)
{
    NitsTrace(L"Group2!");
}

TGROUP(Group3)(Switch &test)
{
    NitsTrace(L"Group3!");
}

TGROUP(Group4)(Switch &test)
{
    NitsTrace(L"Group4!");
}

TCHOICE(ChoiceA)(Switch &test)
{
    NitsTrace(L"ChoiceA!");
    test.Child(Group1);
    test.Child(Group2);
}

TCHOICE(ChoiceB)(Switch &test)
{
    NitsTrace(L"ChoiceB!");
    test.Child(Group3);
}

TCHOICE(ChoiceC)(Switch &test)
{
    NitsTrace(L"ChoiceC!");
    test.Child(Group3);
    test.Child(Group4);
}

TCHOICE(ChoiceD)(Switch &test)
{
    NitsTrace(L"ChoiceD!");
    test.Child(ChoiceB);
    test.Child(ChoiceC);
}

void TestHelper(NitsCallSite site)
{
    NitsTraceEx(L"Helper function!", site, NitsAutomatic);
}

TSETUP(ChoiceTest, Body)(Switch &test)
{
    NitsTrace(L"Setup!");
    test.Child(ChoiceA);
    test.Child(ChoiceD);
}

TSET_ISOLATION(FailedAsserts);
TTEST(FailedAsserts)(Switch &test)
{
    NitsTrace(L"This test deliberately fails asserts.");
    NitsAssert(!true, L"assert should fail!");
    NitsCompare(1, 2, L"compare should fail!");
    NitsCompareString(L"A", L"B", L"compare should fail!");
    NitsCompareSubstring(L"ABC", L"D", L"substring should fail!");

    TestHelper(NitsHere());
}
