#ifndef _pal_atomic_h
#define _pal_atomic_h

#include "palcommon.h"

PAL_BEGIN_EXTERNC

PAL_INLINE ptrdiff_t Atomic_Inc(
    _Inout_ volatile ptrdiff_t* x)
{
#if defined(_MSC_VER)
# if defined(_WIN64)
    return InterlockedIncrement64(x);
# else
    return InterlockedIncrement(x);
# endif
#else
    return __sync_add_and_fetch(x, 1);
#endif
}

PAL_INLINE ptrdiff_t Atomic_Dec(
    _Inout_ volatile ptrdiff_t* x)
{
#if defined(_MSC_VER)
# if defined(_WIN64)
    return InterlockedDecrement64(x);
# else
    return InterlockedDecrement(x);
# endif
#else
    return __sync_sub_and_fetch(x, 1);
#endif
}

PAL_INLINE ptrdiff_t Atomic_CompareAndSwap(
    _Inout_ volatile ptrdiff_t* dest,
    _In_ ptrdiff_t comparand,
    _In_ ptrdiff_t newValue)
{
#if defined(_MSC_VER)
    return (ptrdiff_t)InterlockedCompareExchangePointer(
        (PVOID*)dest, (PVOID)newValue, (PVOID)comparand);
#else
    return __sync_val_compare_and_swap(dest, comparand, newValue);
#endif
}

PAL_INLINE ptrdiff_t Atomic_Swap(
    _Inout_ volatile ptrdiff_t* dest,
    _In_ ptrdiff_t newValue)
{
#if defined(_MSC_VER)
    return (ptrdiff_t)InterlockedExchangePointer((PVOID*)dest, (PVOID)newValue);
#else
    return __sync_lock_test_and_set(dest, newValue);
#endif
}

PAL_INLINE void Atomic_Lock(
    _Inout_ volatile ptrdiff_t* dest)
{
#if defined(_MSC_VER)
# if defined(_WIN64)
    while (InterlockedExchange64(dest, 1))
        while (*dest)
            ;
# else
    while (InterlockedExchange(dest, 1))
        while (*dest)
            ;
# endif
#else
    while (__sync_lock_test_and_set(dest, 1))
        while (*dest)
            ;
#endif
}

PAL_INLINE void Atomic_Unlock(
    _Inout_ volatile ptrdiff_t* dest)
{
#if defined(_MSC_VER)
# if defined(_WIN64)
    InterlockedExchange64(dest, 0);
# else
    InterlockedExhcange(dest, 0);
# endif
#else
    __sync_lock_release(dest);
#endif
}

PAL_END_EXTERNC

#endif /* _pal_atomic_h */
