#ifndef _pal_cond_h
#define _pal_cond_h

#include "palcommon.h"
#include "mutex.h"

#if defined(PAL_HAVE_PTHREADS)
# include <pthread.h>
#endif

/* ATTN: obsolete this module */

PAL_BEGIN_EXTERNC

/*
**==============================================================================
**
** Windows Version
**
**==============================================================================
*/

#if defined(_MSC_VER)

typedef struct _WaitCaller WaitCaller;

typedef struct _Cond
{
    HANDLE mutex;
    WaitCaller* free;
    size_t freeListSize;
    WaitCaller* head;
    WaitCaller* tail;
}
Cond;

void Cond_Init(
    _Out_ Cond* self);

void Cond_Destroy(
    _Inout_ Cond* self);

void Cond_Wait(
    _Inout_ Cond* self,
    _Inout_ Mutex* mutex);

void Cond_Signal(
    _Inout_ Cond* self);

#endif /* defined(_MSC_VER) */

/*
**==============================================================================
**
** POSIX Threads Version
**
**==============================================================================
*/

#if defined(PAL_HAVE_PTHREADS)

typedef struct _Cond
{
    pthread_cond_t cond;
}
Cond;

PAL_INLINE void Cond_Init(
    _Out_ Cond* self)
{
    pthread_cond_init(&self->cond, NULL);
}

PAL_INLINE void Cond_Destroy(
    _Inout_ Cond* self)
{
    pthread_cond_destroy(&self->cond);
}

PAL_INLINE void Cond_Wait(
    _Inout_ Cond* self,
    _Inout_ Mutex* mutex)
{
    pthread_cond_wait(&self->cond, &mutex->mutex);
}

PAL_INLINE void Cond_Signal(
    _Inout_ Cond* self)
{
    pthread_cond_signal(&self->cond);
}

#endif /* defined(PAL_HAVE_PTHREADS) */

PAL_END_EXTERNC

#endif /* _pal_cond_h */
