#ifndef _pal_cpu_h
#define _pal_cpu_h

#include <stddef.h>
#include "palcommon.h"

#if defined(PAL_HAVE_POSIX)
# include <unistd.h>
# include <time.h>
# include <sys/time.h>
# include <sys/types.h>
#endif

PAL_BEGIN_EXTERNC

PAL_INLINE int CPU_GetCount()
{
#if defined(_MSC_VER)
    SYSTEM_INFO info;
    GetSystemInfo(&info);
    return info.dwNumberOfProcessors;
#else
    return sysconf(_SC_NPROCESSORS_ONLN);
#endif
}

PAL_INLINE int CPU_GetCurrent()
{
#if defined(_MSC_VER)
    return GetCurrentProcessorNumber();
#else
    int sched_getcpu();
    return sched_getcpu();
#endif
}

PAL_INLINE PAL_Uint64 CPU_GetTimeStamp()
{
#if defined(_MSC_VER) && defined(_ARM)
    return GetTickCount64() * 1000;
#elif defined(_MSC_VER)
    return (PAL_Uint64)ReadTimeStampCounter() >> 11;
#else
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return (PAL_Uint64)tv.tv_sec * (PAL_Uint64)1000000 + (PAL_Uint64)tv.tv_usec;
#endif
}

PAL_END_EXTERNC

#endif /* _pal_cpu_h */
