#ifndef _pal_lock_h
#define _pal_lock_h

#include <stddef.h>
#include "palcommon.h"

/*
**==============================================================================
**
** Lock -- simple non-recursive lock
**
**==============================================================================
*/

#define LOCK_INITIALIZER { 0 }

typedef struct _Lock Lock;

void Lock_Init(
    __out Lock* self);

void Lock_Destroy(
    __inout Lock* self);

void Lock_Acquire(
    __inout Lock* self);

void Lock_Release(
    __inout Lock* self);

/*
**==============================================================================
**
** RecursiveLock -- recursive lock
**
**==============================================================================
*/

#define RECURSIVELOCK_INITIALIZER { 0 }

typedef struct _RecursiveLock RecursiveLock;

void RecursiveLock_Init(
    __out RecursiveLock* self);

void RecursiveLock_Destroy(
    __inout RecursiveLock* self);

void RecursiveLock_Acquire(
    __inout RecursiveLock* self);

void RecursiveLock_Release(
    __inout RecursiveLock* self);

/*
**==============================================================================
**
** ReadWriteLock -- read/write lock (non-recursize)
**
**==============================================================================
*/

#define READWRITELOCK_INITIALIZER { 0 }

typedef struct _ReadWriteLock ReadWriteLock;

void ReadWriteLock_Init(
    __out ReadWriteLock* self);

void ReadWriteLock_Destroy(
    __inout ReadWriteLock* self);

void ReadWriteLock_AcquireRead(
    __inout ReadWriteLock* self);

void ReadWriteLock_AcquireWrite(
    __inout ReadWriteLock* self);

void ReadWriteLock_ReleaseRead(
    __inout ReadWriteLock* self);

void ReadWriteLock_ReleaseWrite(
    __inout ReadWriteLock* self);


/*
**==============================================================================
**
** CachedLock -- cached lock
**
**==============================================================================
*/

typedef struct _CachedLock CachedLock;

#define CACHEDLOCK_FLAG_SHARED (1 << 0)

void CachedLock_Init(
    __in unsigned long flags,
    __out CachedLock* self);

void CachedLock_Destroy(
    __inout CachedLock* self);

void CachedLock_AcquireRead(
    __inout CachedLock* self,
    __out void** token);

void CachedLock_AcquireWrite(
    __inout CachedLock* self);

void CachedLock_ReleaseRead(
    __inout CachedLock* self,
    __in void* token);

void CachedLock_ReleaseWrite(
    __inout CachedLock* self);

/*
**==============================================================================
**
** Cond -- condition variable
**
**==============================================================================
*/

#define COND_DEFAULT_SPINCOUNT ((size_t)-1)

#define COND_FLAG_WAKE_SPINNERS_ONLY (1 << 0)

int Cond_Wait(
    __in ptrdiff_t key,
    __in ptrdiff_t volatile const* destination,
    __in ptrdiff_t comparand,
    __in size_t spinCount);

void Cond_Broadcast(
    __in ptrdiff_t key,
    __in unsigned int flags);

void Cond_Signal(
    __in ptrdiff_t key,
    __in unsigned int flags);

#endif /* _pal_lock_h */
