#ifndef _pal_mutex_h
#define _pal_mutex_h

#include <stddef.h>
#include "palcommon.h"

#if defined(PAL_HAVE_PTHREADS)
# include <pthread.h>
#endif

PAL_BEGIN_EXTERNC

/* ATTN: consider obsoleting once locks are available */

/*
**==============================================================================
**
** Mutex -- simple system mutex
**
**==============================================================================
*/

#if defined(_MSC_VER)
# define MUTEX_INITIALIZER { NULL }
#elif defined(PAL_HAVE_PTHREADS)
# define MUTEX_INITIALIZER { PTHREAD_MUTEX_INITIALIZER }
#endif

typedef struct _Mutex
{
#if defined(_MSC_VER)
    HANDLE handle;
#else
    pthread_mutex_t mutex;
#endif
}
Mutex;

void __Mutex_InitStatic(
    _Out_ Mutex* self);

PAL_INLINE void Mutex_Init(
    _Out_ Mutex* self)
{
#if defined(_MSC_VER)
    self->handle = CreateMutex(NULL, FALSE, NULL);
#else
    pthread_mutex_init(&self->mutex, NULL);
#endif
}

PAL_INLINE void Mutex_Destroy(
    _Inout_ Mutex* self)
{
#if defined(_MSC_VER)
    CloseHandle(self->handle);
#else
    pthread_mutex_destroy(&self->mutex);
#endif
}

PAL_INLINE void Mutex_Acquire(
    _Inout_ Mutex* self)
{
#if defined(_MSC_VER)
    if (self->handle == NULL)
        __Mutex_InitStatic(self);
    WaitForSingleObject(self->handle, INFINITE);
#else
    pthread_mutex_lock(&self->mutex);
#endif
}

PAL_INLINE void Mutex_Release(
    _Inout_ Mutex* self)
{
#if defined(_MSC_VER)
    ReleaseMutex(self->handle);
#else
    pthread_mutex_unlock(&self->mutex);
#endif
}

PAL_END_EXTERNC

#endif /* _pal_mutex_h */
