#include "sem.h"
#include <stdio.h>

/*
**==============================================================================
**
** Sem
**
**==============================================================================
*/

int Sem_Init(
    _Out_ Sem* self,
    unsigned int count)
{
#if defined(_MSC_VER)

    self->handle = CreateSemaphore(
        NULL,
        count,
        0xFFFF,
        NULL);

    if (!self->handle)
        return -1;

    return 0;

#else
    return sem_init(&self->sem, 0, count) == 0 ? 0 : -1;
#endif
}

int Sem_Post(
    _Inout_ Sem* self,
    unsigned int count)
{
#if defined(_MSC_VER)
    return ReleaseSemaphore(self->handle, count, NULL) ? 0 : -1;
#else
    while (count--)
    {
        if (sem_post(&self->sem) != 0)
            return -1;
    }

    return 0;
#endif
}

/*
**==============================================================================
**
** NamedSem
**
**     (1) On Linux, sem_open("/erp", ...) creates "/dev/shm/sem.erp"
**
**==============================================================================
*/

int NamedSem_Open(
    _Out_ NamedSem* self,
    unsigned int count,
    _In_ const PAL_Char* name,
    _In_ unsigned long flags)
{
#if defined(_MSC_VER)

    self->handle = CreateSemaphore(
        NULL,
        count,
        0xFFFF,
        (PAL_Char*)name);

    if (!self->handle)
        return -1;

    return 0;

#else

    int tflags = 0;

    if (!name)
        return -1;


    if (flags & NAMEDSEM_FLAG_CREATE)
        tflags |= O_CREAT;

    if (flags & NAMEDSEM_FLAG_EXCLUSIVE)
        tflags |= O_EXCL;

    self->sem = sem_open(name, tflags, 0600, count);

    if (!self->sem)
        return -1;

    return 0;

#endif
}

int NamedSem_Post(
    _Inout_ NamedSem* self,
    unsigned int count)
{
#if defined(_MSC_VER)
    return ReleaseSemaphore(self->handle, count, NULL) ? 0 : -1;
#else

    while (count--)
    {
        if (sem_post(self->sem) != 0)
            return -1;
    }

    return 0;

#endif
}
