#ifndef _pal_sem_h
#define _pal_sem_h

#include <stddef.h>
#include "palcommon.h"

#if defined(PAL_HAVE_POSIX)
# include <fcntl.h>
# include <sys/stat.h>
# include <semaphore.h>
#endif

PAL_BEGIN_EXTERNC

/*
**==============================================================================
**
** Sem - semaphore
**
**==============================================================================
*/

typedef struct _Sem
{
#if defined(_MSC_VER)
    HANDLE handle;
#else
    sem_t sem;
#endif
}
Sem;

int Sem_Init(
    _Out_ Sem* self,
    _In_ unsigned int count);

PAL_INLINE void Sem_Destroy(
    _Inout_ Sem* self)
{
#if defined(_MSC_VER)
    CloseHandle(self->handle);
#else
    sem_close(&self->sem);
#endif
}

PAL_INLINE int Sem_Wait(
    _Inout_ Sem* self)
{
#if defined(_MSC_VER)
    return WaitForSingleObject(self->handle, INFINITE) ? 0 : -1;
#else
    return sem_wait(&self->sem) == 0 ? 0 : -1;
#endif
}

int Sem_Post(
    _Inout_ Sem* self,
    unsigned int count);

/*
**==============================================================================
**
** NamedSem - named semaphore
**
**==============================================================================
*/

#define NAMEDSEM_FLAG_CREATE 1
#define NAMEDSEM_FLAG_EXCLUSIVE 2

typedef struct _NamedSem
{
#if defined(_MSC_VER)
    HANDLE handle;
#else
    sem_t* sem;
#endif
}
NamedSem;

int NamedSem_Open(
    _Out_ NamedSem* self,
    unsigned int count,
    _In_ const PAL_Char* name,
    _In_ unsigned long flags);

PAL_INLINE void NamedSem_Close(
    _Inout_ NamedSem* self)
{
#if defined(_MSC_VER)
    CloseHandle(self->handle);
#else
    sem_close(self->sem);
#endif
}

void NamedSem_Close(
    _Inout_ NamedSem* self);

PAL_INLINE int NamedSem_Wait(
    _Inout_ NamedSem* self)
{
#if defined(_MSC_VER)
    return WaitForSingleObject(self->handle, INFINITE) ? 0 : -1;
#else
    return sem_wait(self->sem) == 0 ? 0 : -1;
#endif
}

int NamedSem_Post(
    _Inout_ NamedSem* self,
    unsigned int count);

PAL_END_EXTERNC

#endif /* _pal_sem_h */
