#ifndef _pal_shlib_h
#define _pal_shlib_h

#include <stddef.h>
#include <string.h>
#include "palcommon.h"

#if defined(PAL_HAVE_POSIX)
# include <dlfcn.h>
#endif

PAL_BEGIN_EXTERNC

typedef struct _Shlib Shlib;

PAL_INLINE Shlib* Shlib_Open(
    _In_ const char* path)
{
#if defined(_MSC_VER)
    return (Shlib*)LoadLibraryA(path);
#else
    int flags = RTLD_NOW | RTLD_LOCAL;
    return (Shlib*)dlopen(path, flags);
#endif
}

PAL_INLINE int Shlib_Close(
    _Inout_ Shlib* self)
{
#if defined(_MSC_VER)
    return FreeLibrary((HMODULE)self) ? 0 : -1;
#else
    return (dlclose(self) == 0) ? 0 : -1;
#endif
}

PAL_INLINE void* Shlib_Sym(
    _In_ Shlib* self,
    _In_ const char* symbol)
{
#if defined(_MSC_VER)
    FARPROC result;
    result = GetProcAddress((HMODULE)self, symbol);
    return (void*)result;
#else
    return dlsym(self, symbol);
#endif
}

char* Shlib_Err();

PAL_INLINE void Shlib_FreeErr(
    _In_ char* err)
{
#if defined(_MSC_VER)
    LocalFree(err);
#else
    free(err);
#endif
}

PAL_END_EXTERNC

#endif /* _pal_shlib_h */
