#include "slist.h"

#if !defined(_MSC_VER)

SListEntry* SList_FlushAtomic(
    _Inout_ SListHead* head)
{
    Atomic_Lock(&head->__private.lock);
    SListEntry* p = head->__private.next;
    head->__private.next = NULL;
    Atomic_Unlock(&head->__private.lock);
    return p;
}

SListEntry* SList_PushAtomic(
    _Inout_ SListHead* head,
    _Inout_ SListEntry* entry)
{
    SListEntry* p;

    for (;;)
    {
        p = head->__private.next;
        entry->__private.next = p;

        if (Atomic_CompareAndSwap(
            (ptrdiff_t*)&head->__private.next, 
            (ptrdiff_t)p, 
            (ptrdiff_t)entry) == (ptrdiff_t)p)
        {
            break;
        }
    }

    return p;
}

SListEntry* SList_PopAtomic(
    _Inout_ SListHead* head)
{
    Atomic_Lock(&head->__private.lock);

    SListEntry* p = head->__private.next;

    if (p)
    {
        head->__private.next = p->__private.next;
    }

    Atomic_Unlock(&head->__private.lock);

    return p;
}

#endif /* !defined(_MSC_VER) */
