#ifndef _pal_thread_h
#define _pal_thread_h

#include <stddef.h>
#include <string.h>
#include "palcommon.h"

#if defined(PAL_HAVE_PTHREADS)
# include <pthread.h>
#endif

PAL_BEGIN_EXTERNC

typedef struct _Thread 
{
#if defined(_MSC_VER)
    HANDLE __impl;
#elif defined(PAL_HAVE_PTHREADS)
    pthread_t __impl;
#endif
}
Thread;

typedef struct _ThreadID
{
#if defined(_MSC_VER)
    DWORD __impl;
#elif defined(PAL_HAVE_PTHREADS)
    pthread_t __impl;
#endif
}
ThreadID;

#if defined(_MSC_VER)
# define THREAD_API WINAPI
#else
# define THREAD_API /* empty */
#endif

typedef PAL_Uint32 (THREAD_API *ThreadProc)(void* param);

int Thread_CreateDetached(
    _Out_ ThreadID* self,
    _In_ ThreadProc proc,
    _In_ void* param);

int Thread_CreateJoinable(
    _Out_ Thread* self,
    _In_ ThreadProc proc,
    _In_ void* param);

PAL_INLINE void Thread_Destroy(
    _Inout_ Thread* self)
{
#if defined(_MSC_VER)
    CloseHandle(self->__impl);
#else
    /* Zero out so it will fail if called before join */
    memset(self, 0, sizeof(*self));
#endif
}

void Thread_Exit(
    _In_ PAL_Uint32 returnValue);

int Thread_Join(
    _In_ Thread* self,
    _Out_ PAL_Uint32* returnValue);

int Thread_Equal(
    _In_ ThreadID* thread1,
    _In_ ThreadID* thread2);

void Thread_Yield();

ThreadID Thread_ID();

int Thread_IDToThread(
    _In_ ThreadID* self,
    _Out_ Thread* thread);


PAL_END_EXTERNC

#endif /* _pal_thread_h */
