#ifndef _pal_tls_h
#define _pal_tls_h

#include <stddef.h>
#include "palcommon.h"

#if defined(PAL_HAVE_PTHREADS)
# include <pthread.h>
#endif

PAL_BEGIN_EXTERNC

typedef struct _TLS 
{
#if defined(_MSC_VER)
    DWORD index;
#else
    pthread_key_t key;
#endif
}
TLS;

PAL_INLINE int TLS_Init(
    _Out_ TLS* self)
{
#if defined(_MSC_VER)
    self->index = TlsAlloc();
    return self->index == TLS_OUT_OF_INDEXES ? -1 : 0;
#else
    return pthread_key_create(&self->key, NULL) == 0 ? 0 : -1;
#endif
}

PAL_INLINE int TLS_Destroy(
    _Inout_ TLS* self)
{
#if defined(_MSC_VER)
    return TlsFree(self->index) ? 0 : -1;
#else
    return pthread_key_delete(self->key) == 0 ? 0 : -1;
#endif
}

PAL_INLINE int TLS_Set(
    _Inout_ TLS* self,
    _In_ ptrdiff_t data)
{
#if defined(_MSC_VER)
    return TlsSetValue(self->index, (void*)data) ? 0 : -1;
#else
    return pthread_setspecific(self->key, (void*)data) == 0 ? 0 : -1;
#endif
}

PAL_INLINE ptrdiff_t TLS_Get(
    _In_ TLS* self)
{
#if defined(_MSC_VER)
    return (ptrdiff_t)TlsGetValue(self->index);
#else
    return (ptrdiff_t)pthread_getspecific(self->key);
#endif
}

PAL_END_EXTERNC

#endif /* _pal_tls_h */
